// Demo Of Dynamic Multi-Dimensional Templated Array Class Using String Class
// cl Demo26.cpp Strings.cpp /O1 /Os /GS- /Zc:sizedDealloc- TCLib.lib kernel32.lib user32.lib    // <<< For Use With TCLib.lib
// cl Demo26.cpp Strings.cpp /O1 /Os /GS- /EHs kernel32.lib user32.lib                           // <<< For Standard C Runtime Linkage
// 145,408 Bytes C Runtime Linkage;   x64; VC19 (VS 2015); UNICODE
//   6,144 Bytes TCLib Linkage;       x64; VC15 (VS 2008); UNICODE
#define TCLib
#ifndef UNICODE
   #define UNICODE
#endif
#ifndef _UNICODE
   #define _UNICODE
#endif
#include <windows.h>
#ifdef TCLib
   #include "stdio.h"            // Since Microsoft Recently 'Refactored' The C Runtime, I Can't Use Their Includes With TCLib Anymore
   #include "stdlib.h"           // They've Apparently Added Some New function descriptors that screw things up for me.
   #define NEW new               // My newdel.cpp file has alternate definitions (simpler) of new and new []
#else
   #include <cstdio>
   #include <cstdlib>
   #include <new>
   #define NEW new(std::nothrow)
#endif
#include "Strings.h"             // My String Class


template <class datatype> class CArray            // This entity is a templated class for creating dynamic
{                                                 // multi-dimensional arrays of from one to four dimensions.
 public:                                          // It allows for a basic language type syntax for doing
 CArray() : pObjs(0)                              // the above.  GNU C++ compilers implement dynamic array
 {                                                // allocation but MS VC++ compilers don't.  Since I wanted
  this->iNumObjects=0;                            // to test compile with both I developed this class.
  d1=d2=d3=d4=0;
 }

 CArray(int i) : pObjs(0)                         // One Dimensional Array Constructor
 {
  this->iNumObjects=i;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=0, d3=0, d4=0;
 }

 CArray(int i, int j) : pObjs(0)                  // Two Dimensional Array Constructor
 {
  this->iNumObjects=i*j;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=0, d4=0;
 }

 CArray(int i, int j, int k) : pObjs(0)           // Three Dimensional Array Constructor
 {
  this->iNumObjects=i*j*k;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=0;
 }

 CArray(int i, int j, int k, int l) : pObjs(0)    // Four Dimensional Array Constructor
 {
  this->iNumObjects=i*j*k*l;
  this->pObjs = NEW datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=l;
 }

 datatype& operator()(int i)                      // One Dimensional Accessor
 {
  return pObjs[i];
 }

 datatype& operator()(int i, int j)               // Two Dimensional Accessor
 {
  return pObjs[i*d2 + j];
 }

 datatype& operator()(int i, int j, int k)        // Three Dimensional Accessor
 {
  return pObjs[i*d2 + j + k*d1*d2];
 }

 datatype& operator()(int i, int j, int k, int l) // Four Dimensional Accessor
 {
  return pObjs[i*d2 + j + k*d1*d2 + l*d1*d2*d3];
 }

 bool blnMemoryIsGood()
 {
  return !!pObjs;
 }

 int UBound(int iDim)
 {
  if(iDim==1)
     return d1-1;
  if(iDim==2)
     return d2-1;
  if(iDim==3)
     return d3-1;
  if(iDim==4)
     return d4-1;
  else
     return 0;
 }

 ~CArray()
 {
  if(this->pObjs)
     delete [] this->pObjs;
 }

 private:
 datatype* pObjs;       // pointer to the base memory allocation for array
 int       iNumObjects; // We'll need this to zero memory for non-class types
 int       d1;          // Dimension #1
 int       d2;          // Dimension #2
 int       d3;          // Dimension #3
 int       d4;          // Dimension #4
};


template<typename t1> void Output(CArray<t1>& Ar3)
{
 for(int h=0; h<=Ar3.UBound(3); h++)
 {
     for(int i=0; i<=Ar3.UBound(1); i++)
     {
         for(int j=0; j<=Ar3.UBound(2); j++)
             wprintf(L"%s\t",Ar3(i,j,h).lpStr());
         printf("\n");
     }
     printf("\n");
 }
}


int main()
{
 int rows=4,cols=5,levels=6,i,j,k;
 wchar_t szBuffer[16];

 CArray<String> ar_3(rows,cols,levels);
 for(i=0; i<=ar_3.UBound(3); i++)
 {
     for(j=0; j<=ar_3.UBound(1); j++)
     {
         for(k=0; k<=ar_3.UBound(2); k++)
         {
             /*                                        // Hard way of doing it as in C
             ar_3(j,k,i)=L"(";
             swprintf(szBuffer,L"%d",j);
             ar_3(j,k,i)=ar_3(j,k,i)+szBuffer+L",";
             swprintf(szBuffer,L"%d",k);
             ar_3(j,k,i)=ar_3(j,k,i)+szBuffer+L",";
             swprintf(szBuffer,L"%d",i);
             ar_3(j,k,i)=ar_3(j,k,i)+szBuffer+L")";
             */

             /*                                        // Easier way of doing it with C++ String Class
             String s1(j);
             String s2(k);
             String s3(i);
             ar_3(j,k,i)=L"("+s1+L","+s2+L","+s3+L")";
             */

             ar_3(j,k,i) = L"(" + Str(j) + L"," + Str(k) + L"," + Str(i) + L")"; // Best Of All!
          }
     }
 }
 Output(ar_3);
 getchar();

 return 0;
}


#if 0

(0,0,0) (0,1,0) (0,2,0) (0,3,0) (0,4,0)
(1,0,0) (1,1,0) (1,2,0) (1,3,0) (1,4,0)
(2,0,0) (2,1,0) (2,2,0) (2,3,0) (2,4,0)
(3,0,0) (3,1,0) (3,2,0) (3,3,0) (3,4,0)

(0,0,1) (0,1,1) (0,2,1) (0,3,1) (0,4,1)
(1,0,1) (1,1,1) (1,2,1) (1,3,1) (1,4,1)
(2,0,1) (2,1,1) (2,2,1) (2,3,1) (2,4,1)
(3,0,1) (3,1,1) (3,2,1) (3,3,1) (3,4,1)

(0,0,2) (0,1,2) (0,2,2) (0,3,2) (0,4,2)
(1,0,2) (1,1,2) (1,2,2) (1,3,2) (1,4,2)
(2,0,2) (2,1,2) (2,2,2) (2,3,2) (2,4,2)
(3,0,2) (3,1,2) (3,2,2) (3,3,2) (3,4,2)

(0,0,3) (0,1,3) (0,2,3) (0,3,3) (0,4,3)
(1,0,3) (1,1,3) (1,2,3) (1,3,3) (1,4,3)
(2,0,3) (2,1,3) (2,2,3) (2,3,3) (2,4,3)
(3,0,3) (3,1,3) (3,2,3) (3,3,3) (3,4,3)

(0,0,4) (0,1,4) (0,2,4) (0,3,4) (0,4,4)
(1,0,4) (1,1,4) (1,2,4) (1,3,4) (1,4,4)
(2,0,4) (2,1,4) (2,2,4) (2,3,4) (2,4,4)
(3,0,4) (3,1,4) (3,2,4) (3,3,4) (3,4,4)

(0,0,5) (0,1,5) (0,2,5) (0,3,5) (0,4,5)
(1,0,5) (1,1,5) (1,2,5) (1,3,5) (1,4,5)
(2,0,5) (2,1,5) (2,2,5) (2,3,5) (2,4,5)
(3,0,5) (3,1,5) (3,2,5) (3,3,5) (3,4,5)

#endif


/*
// cl Demo26.cpp /O1 /Os /MT /GS- /EHs
#include <cstdio>
#include <cstdlib>
#include <string>
// 143,872 Bytes C++ Standard Library String Class

template <class datatype> class CArray            // This entity is a templated class for creating dynamic
{                                                 // multi-dimensional arrays of from one to four dimensions.
 public:                                          // It allows for a basic language type syntax for doing
 CArray() : pObjs(0)                              // the above.  GNU C++ compilers implement dynamic array
 {                                                // allocation but MS VC++ compilers don't.  Since I wanted
  this->iNumObjects=0;                            // to test compile with both I developed this class.
  d1=d2=d3=d4=0;
 }

 CArray(int i) : pObjs(0)                         // One Dimensional Array Constructor
 {
  this->iNumObjects=i;
  this->pObjs=new(std::nothrow) datatype[this->iNumObjects]();
  d1=i, d2=0, d3=0, d4=0;
 }

 CArray(int i, int j) : pObjs(0)                  // Two Dimensional Array Constructor
 {
  this->iNumObjects=i*j;
  this->pObjs=new(std::nothrow) datatype[this->iNumObjects]();
  d1=i, d2=j, d3=0, d4=0;
 }

 CArray(int i, int j, int k) : pObjs(0)           // Three Dimensional Array Constructor
 {
  this->iNumObjects=i*j*k;
  this->pObjs=new(std::nothrow) datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=0;
 }

 CArray(int i, int j, int k, int l) : pObjs(0)    // Four Dimensional Array Constructor
 {
  this->iNumObjects=i*j*k*l;
  this->pObjs=new(std::nothrow) datatype[this->iNumObjects]();
  d1=i, d2=j, d3=k, d4=l;
 }

 datatype& operator()(int i)                      // One Dimensional Accessor
 {
  return pObjs[i];
 }

 datatype& operator()(int i, int j)               // Two Dimensional Accessor
 {
  return pObjs[i*d2 + j];
 }

 datatype& operator()(int i, int j, int k)        // Three Dimensional Accessor
 {
  return pObjs[i*d2 + j + k*d1*d2];
 }

 datatype& operator()(int i, int j, int k, int l) // Four Dimensional Accessor
 {
  return pObjs[i*d2 + j + k*d1*d2 + l*d1*d2*d3];
 }

 bool blnMemoryIsGood()
 {
  return !!pObjs;
 }

 int UBound(int iDim)
 {
  if(iDim==1)
     return d1-1;
  if(iDim==2)
     return d2-1;
  if(iDim==3)
     return d3-1;
  if(iDim==4)
     return d4-1;
  else
     return 0;
 }

 ~CArray()
 {
  if(this->pObjs)
     delete [] this->pObjs;
 }

 private:
 datatype* pObjs;       // pointer to the base memory allocation for array
 int       iNumObjects; // We'll need this to zero memory for non-class types
 int       d1;          // Dimension #1
 int       d2;          // Dimension #2
 int       d3;          // Dimension #3
 int       d4;          // Dimension #4
};


template<typename t1> void Output(CArray<t1>& Ar3)
{
 for(int h=0; h<=Ar3.UBound(3); h++)
 {
     for(int i=0; i<=Ar3.UBound(1); i++)
     {
         for(int j=0; j<=Ar3.UBound(2); j++)
             wprintf(L"%s\t",Ar3(i,j,h).c_str());
         wprintf(L"\n");
     }
     wprintf(L"\n");
 }
}


int main()
{
 int rows=4,cols=5,levels=6,i,j,k;
 wchar_t szBuffer[16];

 CArray<std::wstring> ar_3(rows,cols,levels);
 for(i=0; i<=ar_3.UBound(3); i++)
 {
     for(j=0; j<=ar_3.UBound(1); j++)
     {
         for(k=0; k<=ar_3.UBound(2); k++)
         {
             ar_3(j,k,i)=L"(";
             swprintf(szBuffer,L"%d",j);
             ar_3(j,k,i)=ar_3(j,k,i)+szBuffer+L",";
             swprintf(szBuffer,L"%d",k);
             ar_3(j,k,i)=ar_3(j,k,i)+szBuffer+L",";
             swprintf(szBuffer,L"%d",i);
             ar_3(j,k,i)=ar_3(j,k,i)+szBuffer+L")";
         }
     }
 }
 Output(ar_3);
 getchar();

 return 0;
}
*/

/*
(0,0,0) (0,1,0) (0,2,0) (0,3,0) (0,4,0)
(1,0,0) (1,1,0) (1,2,0) (1,3,0) (1,4,0)
(2,0,0) (2,1,0) (2,2,0) (2,3,0) (2,4,0)
(3,0,0) (3,1,0) (3,2,0) (3,3,0) (3,4,0)

(0,0,1) (0,1,1) (0,2,1) (0,3,1) (0,4,1)
(1,0,1) (1,1,1) (1,2,1) (1,3,1) (1,4,1)
(2,0,1) (2,1,1) (2,2,1) (2,3,1) (2,4,1)
(3,0,1) (3,1,1) (3,2,1) (3,3,1) (3,4,1)

(0,0,2) (0,1,2) (0,2,2) (0,3,2) (0,4,2)
(1,0,2) (1,1,2) (1,2,2) (1,3,2) (1,4,2)
(2,0,2) (2,1,2) (2,2,2) (2,3,2) (2,4,2)
(3,0,2) (3,1,2) (3,2,2) (3,3,2) (3,4,2)

(0,0,3) (0,1,3) (0,2,3) (0,3,3) (0,4,3)
(1,0,3) (1,1,3) (1,2,3) (1,3,3) (1,4,3)
(2,0,3) (2,1,3) (2,2,3) (2,3,3) (2,4,3)
(3,0,3) (3,1,3) (3,2,3) (3,3,3) (3,4,3)

(0,0,4) (0,1,4) (0,2,4) (0,3,4) (0,4,4)
(1,0,4) (1,1,4) (1,2,4) (1,3,4) (1,4,4)
(2,0,4) (2,1,4) (2,2,4) (2,3,4) (2,4,4)
(3,0,4) (3,1,4) (3,2,4) (3,3,4) (3,4,4)

(0,0,5) (0,1,5) (0,2,5) (0,3,5) (0,4,5)
(1,0,5) (1,1,5) (1,2,5) (1,3,5) (1,4,5)
(2,0,5) (2,1,5) (2,2,5) (2,3,5) (2,4,5)
(3,0,5) (3,1,5) (3,2,5) (3,3,5) (3,4,5)
*/

